"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GatewayManager = void 0;
const exceptions_1 = require("./exceptions");
const gateway_1 = require("./gateway");
const ioredis_1 = __importDefault(require("ioredis"));
class GatewayManager {
    constructor(PaymentMethods, Options = {}) {
        this.PaymentMethods = PaymentMethods;
        this.Options = Options;
        this.ExecutedBlacklist = [];
        this.blacklistPaymentID = async (id, time = 259200 * 1000) => {
            if (this.Redis)
                await this.Redis.set(`epic-pay:executedPayments:${id}`, id, "PX", time);
            else {
                // Clear Old Record
                this.ExecutedBlacklist = this.ExecutedBlacklist.filter((item) => item.timestamp > Date.now() - time // Default 3 Days
                );
                // Add New Token
                this.ExecutedBlacklist.push({
                    id,
                    timestamp: Date.now(),
                });
            }
            return this;
        };
        this.paymentIsExecuted = async (id) => {
            return !!(this.Redis
                ? await this.Redis.exists(`epic-pay:executedPayments:${id}`)
                : this.ExecutedBlacklist.filter((item) => item.id === id).length);
        };
        this.unBlacklistPaymentID = async (id) => {
            if (this.Redis)
                await this.Redis.del(`epic-pay:executedPayments:${id}`);
            else
                this.ExecutedBlacklist = this.ExecutedBlacklist.filter((item) => item.id !== id);
            return this;
        };
        this.listMethods = (currency) => {
            const Methods = Object.keys(this.PaymentMethods);
            return currency
                ? Object.values(this.PaymentMethods)
                    .map((method, index) => (method.currencies || []).includes(currency)
                    ? Methods[index]
                    : undefined)
                    .filter(Boolean)
                : Methods;
        };
        this.get = (method, mode) => {
            var _a;
            if (typeof this.PaymentMethods[method] === "object")
                return new gateway_1.Gateway(method, undefined, this).configure(Object.assign(Object.assign({}, this.PaymentMethods[method]), { mode: mode || ((_a = this.PaymentMethods[method]) === null || _a === void 0 ? void 0 : _a.mode) || "Live" }));
            else
                throw new exceptions_1.GatewayException(`No Configuration has been provided for method '${method}'!`);
        };
        this.PaymentMethods = Object.keys(this.PaymentMethods).reduce((options, method) => typeof this.PaymentMethods[method] === "object"
            ? Object.assign(Object.assign({}, options), { [method]: this.PaymentMethods[method] }) : options, {});
        // Create Redis Client
        if (this.Options.redis instanceof ioredis_1.default)
            this.Redis = this.Options.redis;
        else if (typeof this.Options.redis === "object")
            this.Redis = new ioredis_1.default(this.Options.redis);
    }
}
exports.GatewayManager = GatewayManager;
